import {Injectable} from "@angular/core";
import ApiService, {AvailableVersion} from "@api/api.service";
import {DateOnly} from "@model/dateOnly";
import {PairPeriodTime} from "@model/pairPeriodTime";
import {ScheduleRequest} from "@api/v1/scheduleRequest";
import {ScheduleResponse} from "@api/v1/scheduleResponse";
import {map} from "rxjs";

@Injectable()
export class ScheduleService extends ApiService {
  public readonly basePath = 'Schedule/';
  public readonly version = AvailableVersion.v1;

  public startTerm() {
    return this.get<string>('StartTerm').pipe(map(date => new DateOnly(date)));
  }

  public pairPeriod() {
    return this.get<PairPeriodTime>('PairPeriod');
  }

  public postSchedule(data: ScheduleRequest) {
    let request = this.createRequestBuilder()
      .setData(data)
      .build;

    return this.post<ScheduleResponse[]>(request);
  }

  public getByGroup(id: number, isEven: boolean | null = null, disciplines: Array<number> | null = null, professors: Array<number> | null = null, lectureHalls: Array<number> | null = null) {
    let request = this.createRequestBuilder()
      .setEndpoint('GetByGroup/' + id.toString())
      .setQueryParams({isEven: isEven, disciplines: disciplines, professors: professors, lectureHalls: lectureHalls})
      .build;

    return this.get<ScheduleResponse[]>(request);
  }

  public getByProfessor(id: number, isEven: boolean | null = null, disciplines: Array<number> | null = null, groups: Array<number> | null = null, lectureHalls: Array<number> | null = null) {
    let request = this.createRequestBuilder()
      .setEndpoint('GetByProfessor/' + id.toString())
      .setQueryParams({isEven: isEven, disciplines: disciplines, groups: groups, lectureHalls: lectureHalls})
      .build;

    return this.get<ScheduleResponse[]>(request);
  }

  public getByLectureHall(id: number, isEven: boolean | null = null, disciplines: Array<number> | null = null, groups: Array<number> | null = null, professors: Array<number> | null = null) {
    let request = this.createRequestBuilder()
      .setEndpoint('GetByLectureHall/' + id.toString())
      .setQueryParams({isEven: isEven, disciplines: disciplines, groups: groups, professors: professors})
      .build;

    return this.get<ScheduleResponse[]>(request);
  }

  public getByDiscipline(id: number, isEven: boolean | null = null, groups: Array<number> | null = null, professors: Array<number> | null = null, lectureHalls: Array<number> | null = null) {
    let request = this.createRequestBuilder()
      .setEndpoint('GetByDiscipline/' + id.toString())
      .setQueryParams({isEven: isEven, groups: groups, professors: professors, lectureHalls: lectureHalls})
      .build;

    return this.get<ScheduleResponse[]>(request);
  }
}
